@@ -5,11 +5,11 @@ package Jifty;
 use encoding 'utf8';
 # Work around the fact that Time::Local caches thing on first require
 BEGIN { local $ENV{'TZ'} = "GMT";  require Time::Local;}
-our $VERSION = '0.60507';
+$Jifty::VERSION = '0.60615';
 
 =head1 NAME
 
-Jifty -- Just Do It
+Jifty - an application framework
 
 =head1 DESCRIPTION
 
@@ -62,7 +62,7 @@ probably a better place to start.
 use base qw/Jifty::Object/;
 use Jifty::Everything;
 
-use vars qw/$HANDLE $CONFIG $LOGGER $HANDLER $DISPATCHER $API/;
+use vars qw/$HANDLE $CONFIG $LOGGER $HANDLER $API @PLUGINS/;
 
 =head1 METHODS
 
@@ -116,10 +116,18 @@ sub new {
     push @Jifty::Record::ISA, Jifty->config->framework('Database')->{'RecordBaseClass'};
 
     __PACKAGE__->logger( Jifty::Logger->new( $args{'logger_component'} ) );
-   # Get a classloader set up
-   Jifty::ClassLoader->new->require;
+    # Get a classloader set up
+    Jifty::ClassLoader->new(base => Jifty->config->framework('ApplicationClass'))->require;
 
-    __PACKAGE__->dispatcher(Jifty::Dispatcher->new());
+    # Set up plugins
+    my @plugins;
+    for my $plugin (@{Jifty->config->framework('Plugins')}) {
+        my $class = "Jifty::Plugin::".(keys %{$plugin})[0];
+        my %options = %{ $plugin->{(keys %{$plugin})[0]} };
+        Jifty::Util->require($class);
+        push @plugins, $class->new(%options);
+    }
+    __PACKAGE__->plugins(@plugins);
     __PACKAGE__->handler(Jifty::Handler->new());
     __PACKAGE__->api(Jifty::API->new());
 
@@ -181,19 +189,6 @@ sub handle {
     return $HANDLE;
 }
 
-=head2 dispatcher
-
-An accessor for the C<Jifty::Dispatcher> object that we use to make
-decisions about how to dispatch each request made by a web client.
-
-=cut
-
-sub dispatcher {
-    my $class = shift;
-    $DISPATCHER = shift if (@_);
-    return $DISPATCHER;
-}
-
 =head2 api
 
 An accessor for the L<Jifty::API> object that publishes and controls
@@ -218,6 +213,17 @@ sub web {
     return $HTML::Mason::Commands::JiftyWeb;
 }
 
+=head2 plugins
+
+Returns a list of L<Jifty::Plugin> objects for this Jifty application.
+
+=cut
+
+sub plugins {
+    my $class = shift;
+    @PLUGINS = @_ if @_;
+    return @PLUGINS;
+}
 
 =head2 setup_database_connection
 
